package org.ibex.crypto;
import java.math.BigInteger;
import java.util.*;

public class RSA {
    private final BigInteger pq;
    private final BigInteger e;
    private final boolean reverse;
    
    public RSA(BigInteger pq, BigInteger e, boolean reverse) {
        this.pq = pq;
        this.e = e;
        this.reverse = reverse;
    }
    
    public int getInputBlockSize() { return (pq.bitLength()+7) / 8 - (reverse ? 0 : 1); }
    public int getOutputBlockSize() { return (pq.bitLength()+7) / 8 - (reverse ? 1 : 0); }
    
    public byte[] process(byte[] in) {
        // output block is the same size as the modulus (rounded up)
        int outSize = getOutputBlockSize();
        BigInteger t = new BigInteger(1,in);
        BigInteger c = t.modPow(e,pq);
        byte[] cbytes = c.toByteArray();
        if(cbytes.length > outSize || (reverse && cbytes[0] == 0)) {
            if(cbytes[0] != 0) throw new RuntimeException("should never happen");
            byte[] buf = new byte[outSize];
            System.arraycopy(cbytes,1,buf,0,outSize);
            return buf;
        } else if(!reverse && cbytes.length < outSize) {
            // output needs to be exactly outSize in length
            byte[] buf = new byte[outSize];
            System.arraycopy(cbytes,0,buf,outSize-cbytes.length,cbytes.length);
            return buf;
        } else {
            return cbytes;
        }
    }

    public static class PublicKey {
        public final BigInteger modulus;
        public final BigInteger exponent;
        
        public PublicKey(Object o) {
            Vector seq = (Vector) o;
            modulus = (BigInteger) seq.elementAt(0);
            exponent = (BigInteger) seq.elementAt(1);
        }
    }

}
